@@ -7,6 +7,10 @@ Order = namedtuple('Order', ['symbol', 'shares'])
 class Orders(list):
     """ Orders object that an OrderGenerator should return. """
 
+    def __init__(self, *args):
+        super(Orders, self).__init__()
+        self.extend(args)
+
     def add_order(self, symbol, shares):
         """ Add an order to the orders list.
 
